@@ -148,6 +148,7 @@ def register_service(service):
     This mechanism is used to e.g. automatically start ofp_handler if
     there are applications consuming OFP events.
     """
-    frm = inspect.stack()[1]
-    m = inspect.getmodule(frm[0])
+    frame = inspect.currentframe()
+    m_name = frame.f_back.f_globals['__name__']
+    m = sys.modules[m_name]
     m._SERVICE_NAME = service
