@@ -13,19 +13,19 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import base64
 import logging
 
 from ryu.ofproto import ether
-from ryu.ofproto import ofproto_v1_5
-from ryu.ofproto import ofproto_v1_5_parser
+from ryu.ofproto import ofproto_v1_4
+from ryu.ofproto import ofproto_v1_4_parser
 from ryu.lib import ofctl_utils
 
 LOG = logging.getLogger(__name__)
 
 DEFAULT_TIMEOUT = 1.0
 
-UTIL = ofctl_utils.OFCtlUtil(ofproto_v1_5)
+UTIL = ofctl_utils.OFCtlUtil(ofproto_v1_4)
+str_to_int = ofctl_utils.str_to_int
 
 
 def to_action(dp, dic):
@@ -69,16 +69,19 @@ def to_instructions(dp, insts):
             instructions.append(
                 parser.OFPInstructionActions(ofp.OFPIT_CLEAR_ACTIONS, []))
         elif inst_type == 'GOTO_TABLE':
-            table_id = int(i.get('table_id'))
+            table_id = str_to_int(i.get('table_id'))
             instructions.append(parser.OFPInstructionGotoTable(table_id))
         elif inst_type == 'WRITE_METADATA':
-            metadata = ofctl_utils.str_to_int(i.get('metadata'))
-            metadata_mask = (ofctl_utils.str_to_int(i['metadata_mask'])
+            metadata = str_to_int(i.get('metadata'))
+            metadata_mask = (str_to_int(i['metadata_mask'])
                              if 'metadata_mask' in i
                              else parser.UINT64_MAX)
             instructions.append(
                 parser.OFPInstructionWriteMetadata(
                     metadata, metadata_mask))
+        elif inst_type == 'METER':
+            meter_id = str_to_int(i.get('meter_id'))
+            instructions.append(parser.OFPInstructionMeter(meter_id))
         else:
             LOG.error('Unknown instruction type: %s', inst_type)
 
@@ -90,15 +93,11 @@ def action_to_str(act):
     t = UTIL.ofp_action_type_to_user(s['type'])
     s['type'] = t if t != s['type'] else 'UNKNOWN'
 
-    if t == 'SET_FIELD':
+    if 'field' in s:
         field = s.pop('field')
         s['field'] = field['OXMTlv']['field']
         s['mask'] = field['OXMTlv']['mask']
         s['value'] = field['OXMTlv']['value']
-    elif t == 'COPY_FIELD':
-        oxm_ids = s.pop('oxm_ids')
-        s['src_oxm_id'] = oxm_ids[0]['OFPOxmId']['type']
-        s['dst_oxm_id'] = oxm_ids[1]['OFPOxmId']['type']
 
     return s
 
@@ -112,7 +111,7 @@ def instructions_to_str(instructions):
         t = UTIL.ofp_instruction_type_to_user(v['type'])
         inst_type = t if t != v['type'] else 'UNKNOWN'
         # apply/write/clear-action instruction
-        if isinstance(i, ofproto_v1_5_parser.OFPInstructionActions):
+        if isinstance(i, ofproto_v1_4_parser.OFPInstructionActions):
             acts = []
             for a in i.actions:
                 acts.append(action_to_str(a))
@@ -129,50 +128,46 @@ def instructions_to_str(instructions):
 
 def to_match(dp, attrs):
     convert = {'in_port': UTIL.ofp_port_from_user,
-               'in_phy_port': int,
+               'in_phy_port': str_to_int,
                'metadata': ofctl_utils.to_match_masked_int,
                'eth_dst': ofctl_utils.to_match_eth,
                'eth_src': ofctl_utils.to_match_eth,
-               'eth_type': int,
+               'eth_type': str_to_int,
                'vlan_vid': to_match_vid,
-               'vlan_pcp': int,
-               'ip_dscp': int,
-               'ip_ecn': int,
-               'ip_proto': int,
+               'vlan_pcp': str_to_int,
+               'ip_dscp': str_to_int,
+               'ip_ecn': str_to_int,
+               'ip_proto': str_to_int,
                'ipv4_src': ofctl_utils.to_match_ip,
                'ipv4_dst': ofctl_utils.to_match_ip,
-               'tcp_src': int,
-               'tcp_dst': int,
-               'udp_src': int,
-               'udp_dst': int,
-               'sctp_src': int,
-               'sctp_dst': int,
-               'icmpv4_type': int,
-               'icmpv4_code': int,
-               'arp_op': int,
+               'tcp_src': str_to_int,
+               'tcp_dst': str_to_int,
+               'udp_src': str_to_int,
+               'udp_dst': str_to_int,
+               'sctp_src': str_to_int,
+               'sctp_dst': str_to_int,
+               'icmpv4_type': str_to_int,
+               'icmpv4_code': str_to_int,
+               'arp_op': str_to_int,
                'arp_spa': ofctl_utils.to_match_ip,
                'arp_tpa': ofctl_utils.to_match_ip,
                'arp_sha': ofctl_utils.to_match_eth,
                'arp_tha': ofctl_utils.to_match_eth,
                'ipv6_src': ofctl_utils.to_match_ip,
                'ipv6_dst': ofctl_utils.to_match_ip,
-               'ipv6_flabel': int,
-               'icmpv6_type': int,
-               'icmpv6_code': int,
+               'ipv6_flabel': str_to_int,
+               'icmpv6_type': str_to_int,
+               'icmpv6_code': str_to_int,
                'ipv6_nd_target': ofctl_utils.to_match_ip,
                'ipv6_nd_sll': ofctl_utils.to_match_eth,
                'ipv6_nd_tll': ofctl_utils.to_match_eth,
-               'mpls_label': int,
-               'mpls_tc': int,
-               'mpls_bos': int,
+               'mpls_label': str_to_int,
+               'mpls_tc': str_to_int,
+               'mpls_bos': str_to_int,
                'pbb_isid': ofctl_utils.to_match_masked_int,
                'tunnel_id': ofctl_utils.to_match_masked_int,
                'ipv6_exthdr': ofctl_utils.to_match_masked_int,
-               'pbb_uca': int,
-               'tcp_flags': int,
-               'actset_output': int,
-               'packet_type': ofctl_utils.to_match_packet_type,
-               }
+               'pbb_uca': str_to_int}
 
     keys = {'dl_dst': 'eth_dst',
             'dl_src': 'eth_src',
@@ -205,7 +200,7 @@ def to_match(dp, attrs):
 
 
 def to_match_vid(value):
-    return ofctl_utils.to_match_vid(value, ofproto_v1_5.OFPVID_PRESENT)
+    return ofctl_utils.to_match_vid(value, ofproto_v1_4.OFPVID_PRESENT)
 
 
 def match_to_str(ofmatch):
@@ -219,12 +214,9 @@ def match_to_str(ofmatch):
         mask = match_field['OXMTlv']['mask']
         value = match_field['OXMTlv']['value']
         if key == 'vlan_vid':
-            value = ofctl_utils.match_vid_to_str(value, mask,
-                                                 ofproto_v1_5.OFPVID_PRESENT)
+            value = match_vid_to_str(value, mask)
         elif key == 'in_port':
             value = UTIL.ofp_port_to_user(value)
-        elif key == 'packet_type':
-            value = [value >> 16, value & 0xffff]
         else:
             if mask is not None:
                 value = str(value) + '/' + str(mask)
@@ -233,6 +225,11 @@ def match_to_str(ofmatch):
     return match
 
 
+def match_vid_to_str(value, mask):
+    return ofctl_utils.match_vid_to_str(
+        value, mask, ofproto_v1_4.OFPVID_PRESENT)
+
+
 def wrap_dpid_dict(dp, value, to_user=True):
     if to_user:
         return {str(dp.id): value}
@@ -240,31 +237,6 @@ def wrap_dpid_dict(dp, value, to_user=True):
     return {dp.id: value}
 
 
-def stats_to_str(ofstats):
-
-    stats = {}
-    ofstats = ofstats.to_jsondict()['OFPStats']
-    ofstats = ofstats['oxs_fields']
-
-    for s in ofstats:
-        key = s['OXSTlv']['field']
-        if key == 'duration':
-            value = {
-                'duration_sec': s['OXSTlv']['value'][0],
-                'duration_nsec': s['OXSTlv']['value'][1],
-            }
-        elif key == 'idle_time':
-            value = {
-                'idle_time_sec': s['OXSTlv']['value'][0],
-                'idle_time_nsec': s['OXSTlv']['value'][1],
-            }
-        else:
-            value = s['OXSTlv']['value']
-        stats.setdefault(key, value)
-
-    return stats
-
-
 def get_desc_stats(dp, waiters, to_user=True):
     stats = dp.ofproto_parser.OFPDescStatsRequest(dp, 0)
     msgs = []
@@ -343,50 +315,21 @@ def get_queue_desc(dp, waiters, port_no=None, queue_id=None, to_user=True):
     return wrap_dpid_dict(dp, configs, to_user)
 
 
-def get_flow_desc_stats(dp, waiters, flow=None, to_user=True):
-    flow = flow if flow else {}
-    table_id = UTIL.ofp_table_from_user(
-        flow.get('table_id', dp.ofproto.OFPTT_ALL))
-    flags = int(flow.get('flags', 0))
-    out_port = UTIL.ofp_port_from_user(
-        flow.get('out_port', dp.ofproto.OFPP_ANY))
-    out_group = UTIL.ofp_group_from_user(
-        flow.get('out_group', dp.ofproto.OFPG_ANY))
-    cookie = int(flow.get('cookie', 0))
-    cookie_mask = int(flow.get('cookie_mask', 0))
-    match = to_match(dp, flow.get('match', {}))
-
-    stats = dp.ofproto_parser.OFPFlowDescStatsRequest(
-        dp, flags, table_id, out_port, out_group, cookie, cookie_mask,
-        match)
-
-    msgs = []
-    ofctl_utils.send_stats_request(dp, stats, waiters, msgs, LOG)
-
-    flows = []
-    for msg in msgs:
-        for stats in msg.body:
-            s = stats.to_jsondict()[stats.__class__.__name__]
-            s['instructions'] = instructions_to_str(stats.instructions)
-            s['stats'] = stats_to_str(stats.stats)
-            s['match'] = match_to_str(stats.match)
-            flows.append(s)
-
-    return wrap_dpid_dict(dp, flows, to_user)
-
-
 def get_flow_stats(dp, waiters, flow=None, to_user=True):
     flow = flow if flow else {}
     table_id = UTIL.ofp_table_from_user(
         flow.get('table_id', dp.ofproto.OFPTT_ALL))
-    flags = int(flow.get('flags', 0))
+    flags = str_to_int(flow.get('flags', 0))
     out_port = UTIL.ofp_port_from_user(
         flow.get('out_port', dp.ofproto.OFPP_ANY))
     out_group = UTIL.ofp_group_from_user(
         flow.get('out_group', dp.ofproto.OFPG_ANY))
-    cookie = int(flow.get('cookie', 0))
-    cookie_mask = int(flow.get('cookie_mask', 0))
+    cookie = str_to_int(flow.get('cookie', 0))
+    cookie_mask = str_to_int(flow.get('cookie_mask', 0))
     match = to_match(dp, flow.get('match', {}))
+    # Note: OpenFlow does not allow to filter flow entries by priority,
+    # but for efficiency, ofctl provides the way to do it.
+    priority = str_to_int(flow.get('priority', -1))
 
     stats = dp.ofproto_parser.OFPFlowStatsRequest(
         dp, flags, table_id, out_port, out_group, cookie, cookie_mask,
@@ -398,8 +341,11 @@ def get_flow_stats(dp, waiters, flow=None, to_user=True):
     flows = []
     for msg in msgs:
         for stats in msg.body:
+            if 0 <= priority != stats.priority:
+                continue
+
             s = stats.to_jsondict()[stats.__class__.__name__]
-            s['stats'] = stats_to_str(stats.stats)
+            s['instructions'] = instructions_to_str(stats.instructions)
             s['match'] = match_to_str(stats.match)
             flows.append(s)
 
@@ -410,13 +356,13 @@ def get_aggregate_flow_stats(dp, waiters, flow=None, to_user=True):
     flow = flow if flow else {}
     table_id = UTIL.ofp_table_from_user(
         flow.get('table_id', dp.ofproto.OFPTT_ALL))
-    flags = int(flow.get('flags', 0))
+    flags = str_to_int(flow.get('flags', 0))
     out_port = UTIL.ofp_port_from_user(
         flow.get('out_port', dp.ofproto.OFPP_ANY))
     out_group = UTIL.ofp_group_from_user(
         flow.get('out_group', dp.ofproto.OFPG_ANY))
-    cookie = int(flow.get('cookie', 0))
-    cookie_mask = int(flow.get('cookie_mask', 0))
+    cookie = str_to_int(flow.get('cookie', 0))
+    cookie_mask = str_to_int(flow.get('cookie_mask', 0))
     match = to_match(dp, flow.get('match', {}))
 
     stats = dp.ofproto_parser.OFPAggregateStatsRequest(
@@ -430,7 +376,6 @@ def get_aggregate_flow_stats(dp, waiters, flow=None, to_user=True):
     for msg in msgs:
         stats = msg.body
         s = stats.to_jsondict()[stats.__class__.__name__]
-        s['stats'] = stats_to_str(stats.stats)
         flows.append(s)
 
     return wrap_dpid_dict(dp, flows, to_user)
@@ -473,18 +418,12 @@ def get_table_features(dp, waiters, to_user=True):
                       ofproto.OFPTFPT_APPLY_ACTIONS,
                       ofproto.OFPTFPT_APPLY_ACTIONS_MISS]
 
-    p_type_packet = ofproto.OFPTFPT_PACKET_TYPES
-
     p_type_oxms = [ofproto.OFPTFPT_MATCH,
                    ofproto.OFPTFPT_WILDCARDS,
                    ofproto.OFPTFPT_WRITE_SETFIELD,
                    ofproto.OFPTFPT_WRITE_SETFIELD_MISS,
                    ofproto.OFPTFPT_APPLY_SETFIELD,
-                   ofproto.OFPTFPT_APPLY_SETFIELD_MISS,
-                   ofproto.OFPTFPT_WRITE_COPYFIELD,
-                   ofproto.OFPTFPT_WRITE_COPYFIELD_MISS,
-                   ofproto.OFPTFPT_APPLY_COPYFIELD,
-                   ofproto.OFPTFPT_APPLY_COPYFIELD_MISS]
+                   ofproto.OFPTFPT_APPLY_SETFIELD_MISS]
 
     p_type_experimenter = [ofproto.OFPTFPT_EXPERIMENTER,
                            ofproto.OFPTFPT_EXPERIMENTER_MISS]
@@ -501,34 +440,28 @@ def get_table_features(dp, waiters, to_user=True):
                 p['type'] = t if t != prop.type else 'UNKNOWN'
                 if prop.type in p_type_instructions:
                     instruction_ids = []
-                    for id in prop.instruction_ids:
-                        i = {'len': id.len,
-                             'type': id.type}
-                        instruction_ids.append(i)
+                    for i in prop.instruction_ids:
+                        inst = {'len': i.len,
+                                'type': i.type}
+                        instruction_ids.append(inst)
                     p['instruction_ids'] = instruction_ids
                 elif prop.type in p_type_next_tables:
                     table_ids = []
-                    for id in prop.table_ids:
-                        table_ids.append(id)
+                    for i in prop.table_ids:
+                        table_ids.append(i)
                     p['table_ids'] = table_ids
                 elif prop.type in p_type_actions:
                     action_ids = []
-                    for id in prop.action_ids:
-                        i = id.to_jsondict()[id.__class__.__name__]
-                        action_ids.append(i)
+                    for i in prop.action_ids:
+                        act = i.to_jsondict()[i.__class__.__name__]
+                        action_ids.append(act)
                     p['action_ids'] = action_ids
                 elif prop.type in p_type_oxms:
                     oxm_ids = []
-                    for id in prop.oxm_ids:
-                        i = id.to_jsondict()[id.__class__.__name__]
-                        oxm_ids.append(i)
+                    for i in prop.oxm_ids:
+                        oxm = i.to_jsondict()[i.__class__.__name__]
+                        oxm_ids.append(oxm)
                     p['oxm_ids'] = oxm_ids
-                elif prop.type == p_type_packet:
-                    oxm_values = []
-                    for val in prop.oxm_values:
-                        i = {val[0]: val[1]}
-                        oxm_values.append(i)
-                    p['oxm_values'] = oxm_values
                 elif prop.type in p_type_experimenter:
                     pass
                 properties.append(p)
@@ -649,7 +582,7 @@ def get_meter_features(dp, waiters, to_user=True):
     return wrap_dpid_dict(dp, features, to_user)
 
 
-def get_meter_desc(dp, waiters, meter_id=None, to_user=True):
+def get_meter_config(dp, waiters, meter_id=None, to_user=True):
     flags = {dp.ofproto.OFPMF_KBPS: 'KBPS',
              dp.ofproto.OFPMF_PKTPS: 'PKTPS',
              dp.ofproto.OFPMF_BURST: 'BURST',
@@ -660,7 +593,7 @@ def get_meter_desc(dp, waiters, meter_id=None, to_user=True):
     else:
         meter_id = UTIL.ofp_meter_from_user(meter_id)
 
-    stats = dp.ofproto_parser.OFPMeterDescStatsRequest(
+    stats = dp.ofproto_parser.OFPMeterConfigStatsRequest(
         dp, 0, meter_id)
     msgs = []
     ofctl_utils.send_stats_request(dp, stats, waiters, msgs, LOG)
@@ -754,10 +687,7 @@ def get_group_features(dp, waiters, to_user=True):
                    ofp.OFPAT_SET_FIELD: 'SET_FIELD',
                    ofp.OFPAT_PUSH_PBB: 'PUSH_PBB',
                    ofp.OFPAT_POP_PBB: 'POP_PBB',
-                   ofp.OFPAT_COPY_FIELD: 'COPY_FIELD',
-                   ofp.OFPAT_METER: 'METER',
-                   ofp.OFPAT_EXPERIMENTER: 'EXPERIMENTER',
-                   }
+                   ofp.OFPAT_EXPERIMENTER: 'EXPERIMENTER'}
 
     stats = dp.ofproto_parser.OFPGroupFeaturesStatsRequest(dp, 0)
     msgs = []
@@ -818,13 +748,8 @@ def get_group_features(dp, waiters, to_user=True):
     return wrap_dpid_dict(dp, features, to_user)
 
 
-def get_group_desc(dp, waiters, group_id=None, to_user=True):
-    if group_id is None:
-        group_id = dp.ofproto.OFPG_ALL
-    else:
-        group_id = UTIL.ofp_group_from_user(group_id)
-
-    stats = dp.ofproto_parser.OFPGroupDescStatsRequest(dp, 0, group_id)
+def get_group_desc(dp, waiters, to_user=True):
+    stats = dp.ofproto_parser.OFPGroupDescStatsRequest(dp, 0)
     msgs = []
     ofctl_utils.send_stats_request(dp, stats, waiters, msgs, LOG)
 
@@ -842,14 +767,7 @@ def get_group_desc(dp, waiters, group_id=None, to_user=True):
 
                     else:
                         actions.append(action)
-                properties = []
-                for prop in bucket.properties:
-                    p = prop.to_jsondict()[prop.__class__.__name__]
-                    t = UTIL.ofp_group_bucket_prop_type_to_user(prop.type)
-                    p['type'] = t if t != prop.type else 'UNKNOWN'
-                    properties.append(p)
                 b['actions'] = actions
-                b['properties'] = properties
                 buckets.append(b)
 
             d['buckets'] = buckets
@@ -899,28 +817,32 @@ def get_port_desc(dp, waiters, port_no=None, to_user=True):
     return wrap_dpid_dict(dp, descs, to_user)
 
 
+def get_role(dp, waiters, to_user=True):
+    return ofctl_utils.get_role(dp, waiters, to_user)
+
+
 def mod_flow_entry(dp, flow, cmd):
-    cookie = int(flow.get('cookie', 0))
-    cookie_mask = int(flow.get('cookie_mask', 0))
+    cookie = str_to_int(flow.get('cookie', 0))
+    cookie_mask = str_to_int(flow.get('cookie_mask', 0))
     table_id = UTIL.ofp_table_from_user(flow.get('table_id', 0))
-    idle_timeout = int(flow.get('idle_timeout', 0))
-    hard_timeout = int(flow.get('hard_timeout', 0))
-    priority = int(flow.get('priority', 0))
+    idle_timeout = str_to_int(flow.get('idle_timeout', 0))
+    hard_timeout = str_to_int(flow.get('hard_timeout', 0))
+    priority = str_to_int(flow.get('priority', 0))
     buffer_id = UTIL.ofp_buffer_from_user(
         flow.get('buffer_id', dp.ofproto.OFP_NO_BUFFER))
     out_port = UTIL.ofp_port_from_user(
         flow.get('out_port', dp.ofproto.OFPP_ANY))
     out_group = UTIL.ofp_group_from_user(
         flow.get('out_group', dp.ofproto.OFPG_ANY))
-    importance = int(flow.get('importance', 0))
-    flags = int(flow.get('flags', 0))
+    importance = str_to_int(flow.get('importance', 0))
+    flags = str_to_int(flow.get('flags', 0))
     match = to_match(dp, flow.get('match', {}))
     inst = to_instructions(dp, flow.get('instructions', []))
 
     flow_mod = dp.ofproto_parser.OFPFlowMod(
         dp, cookie, cookie_mask, table_id, cmd, idle_timeout,
         hard_timeout, priority, buffer_id, out_port, out_group,
-        importance, flags, match, inst)
+        flags, importance, match, inst)
 
     ofctl_utils.send_msg(dp, flow_mod, LOG)
 
@@ -944,18 +866,18 @@ def mod_meter_entry(dp, meter, cmd):
     bands = []
     for band in meter.get('bands', []):
         band_type = band.get('type')
-        rate = int(band.get('rate', 0))
-        burst_size = int(band.get('burst_size', 0))
+        rate = str_to_int(band.get('rate', 0))
+        burst_size = str_to_int(band.get('burst_size', 0))
         if band_type == 'DROP':
             bands.append(
                 dp.ofproto_parser.OFPMeterBandDrop(rate, burst_size))
         elif band_type == 'DSCP_REMARK':
-            prec_level = int(band.get('prec_level', 0))
+            prec_level = str_to_int(band.get('prec_level', 0))
             bands.append(
                 dp.ofproto_parser.OFPMeterBandDscpRemark(
                     rate, burst_size, prec_level))
         elif band_type == 'EXPERIMENTER':
-            experimenter = int(band.get('experimenter', 0))
+            experimenter = str_to_int(band.get('experimenter', 0))
             bands.append(
                 dp.ofproto_parser.OFPMeterBandExperimenter(
                     rate, burst_size, experimenter))
@@ -969,9 +891,6 @@ def mod_meter_entry(dp, meter, cmd):
 
 
 def mod_group_entry(dp, group, cmd):
-    ofp = dp.ofproto
-    parser = dp.ofproto_parser
-
     group_type = str(group.get('type', 'ALL'))
     t = UTIL.ofp_group_type_from_user(group_type)
     group_type = t if t != group_type else None
@@ -979,73 +898,24 @@ def mod_group_entry(dp, group, cmd):
         LOG.error('Unknown group type: %s', group.get('type'))
 
     group_id = UTIL.ofp_group_from_user(group.get('group_id', 0))
-    command_bucket_id = int(group.get('command_bucket_id', 0))
-
-    # Note:
-    # The list of group property types that are currently defined
-    # are only OFPGPT_EXPERIMENTER(Experimenter defined).
-    properties = []
 
     buckets = []
     for bucket in group.get('buckets', []):
-
-        # get bucket_id in buckets
-        bucket_id = int(bucket.get('bucket_id', 0))
-
-        # get actions in buckets
-        bucket_actions = []
+        weight = str_to_int(bucket.get('weight', 0))
+        watch_port = str_to_int(
+            bucket.get('watch_port', dp.ofproto.OFPP_ANY))
+        watch_group = str_to_int(
+            bucket.get('watch_group', dp.ofproto.OFPG_ANY))
+        actions = []
         for dic in bucket.get('actions', []):
             action = to_action(dp, dic)
             if action is not None:
-                bucket_actions.append(action)
+                actions.append(action)
+        buckets.append(dp.ofproto_parser.OFPBucket(
+            weight, watch_port, watch_group, actions))
 
-        # get properties in buckets
-        bucket_properties = []
-        for p in bucket.get('properties', []):
-            group_bp_type = str(p.get('type', 'WEIGHT'))
-            t = UTIL.ofp_group_bucket_prop_type_from_user(group_bp_type)
-            group_bp_type = t if t != group_bp_type else ofp.OFPGBPT_WEIGHT
-
-            if group_bp_type == ofp.OFPGBPT_WEIGHT:
-                weight = int(p.get('weight', 0))
-                bucket_properties.append(
-                    parser.OFPGroupBucketPropWeight(
-                        type_=group_bp_type, weight=weight))
-            elif group_bp_type == ofp.OFPGBPT_WATCH_PORT:
-                watch_port = int(p.get('watch', dp.ofproto.OFPP_ANY))
-                bucket_properties.append(
-                    parser.OFPGroupBucketPropWatch(
-                        type_=group_bp_type, watch=watch_port))
-            elif group_bp_type == ofp.OFPGBPT_WATCH_GROUP:
-                watch_group = int(p.get('watch', dp.ofproto.OFPG_ANY))
-                bucket_properties.append(
-                    parser.OFPGroupBucketPropWatch(
-                        type_=group_bp_type, watch=watch_group))
-            elif group_bp_type == ofp.OFPGBPT_EXPERIMENTER:
-                experimenter = p.get('experimenter', 0)
-                exp_type = p.get('exp_type', 0)
-                data_type = p.get('data_type', 'ascii')
-                if data_type not in ['ascii', 'base64']:
-                    LOG.error('Unknown data type: %s', data_type)
-                data = p.get('data', '')
-                if data_type == 'base64':
-                    data = base64.b64decode(data)
-                bucket_properties.append(
-                    parser.OFPGroupBucketPropExperimenter(
-                        type_=group_bp_type, experimenter=experimenter,
-                        exp_type=exp_type, data=data))
-            else:
-                LOG.error('Unknown group bucket prop type: %s', p['type'])
-
-        # create bucket
-        bucket = parser.OFPBucket(bucket_id=bucket_id,
-                                  actions=bucket_actions,
-                                  properties=bucket_properties)
-        buckets.append(bucket)
-
-    group_mod = parser.OFPGroupMod(dp, cmd, group_type, group_id,
-                                   command_bucket_id, buckets,
-                                   properties)
+    group_mod = dp.ofproto_parser.OFPGroupMod(
+        dp, cmd, group_type, group_id, buckets)
 
     ofctl_utils.send_msg(dp, group_mod, LOG)
 
@@ -1055,8 +925,8 @@ def mod_port_behavior(dp, port_config):
     parser = dp.ofproto_parser
     port_no = UTIL.ofp_port_from_user(port_config.get('port_no', 0))
     hw_addr = str(port_config.get('hw_addr'))
-    config = int(port_config.get('config', 0))
-    mask = int(port_config.get('mask', 0))
+    config = str_to_int(port_config.get('config', 0))
+    mask = str_to_int(port_config.get('mask', 0))
     properties = port_config.get('properties')
 
     prop = []
@@ -1086,5 +956,11 @@ def mod_port_behavior(dp, port_config):
     ofctl_utils.send_msg(dp, port_mod, LOG)
 
 
+def set_role(dp, role):
+    r = UTIL.ofp_role_from_user(role.get('role', dp.ofproto.OFPCR_ROLE_EQUAL))
+    role_request = dp.ofproto_parser.OFPRoleRequest(dp, r, 0)
+    ofctl_utils.send_msg(dp, role_request, LOG)
+
+
 # NOTE(jkoelker) Alias common funcitons
 send_experimenter = ofctl_utils.send_experimenter
