@@ -1,19 +1,29 @@
 from __future__ import absolute_import
 
-from .route_formatter_mixin import RouteFormatterMixin
-
+from ryu.services.protocols.bgp.base import ActivityException
 from ryu.services.protocols.bgp.operator.command import Command
 from ryu.services.protocols.bgp.operator.command import CommandsResponse
 from ryu.services.protocols.bgp.operator.command import STATUS_ERROR
 from ryu.services.protocols.bgp.operator.command import STATUS_OK
-
-from ryu.services.protocols.bgp.base import ActivityException
-from ryu.services.protocols.bgp.operator.commands.responses import \
-    WrongParamResp
+from ryu.services.protocols.bgp.operator.commands.responses import (
+    WrongParamResp)
+from .route_formatter_mixin import RouteFormatterMixin
 
 
 class RibBase(Command, RouteFormatterMixin):
-    supported_families = ['ipv4', 'ipv6', 'vpnv4', 'rtfilter', 'vpnv6']
+    supported_families = [
+        'ipv4',
+        'ipv6',
+        'vpnv4',
+        'vpnv6',
+        'rtfilter',
+        'evpn',
+        'ipv4fs',
+        'ipv6fs',
+        'vpnv4fs',
+        'vpnv6fs',
+        'l2vpnfs',
+    ]
 
 
 class Rib(RibBase):
