@@ -54,6 +54,8 @@ from ryu.lib.packet.bgp import RF_IPv4_UC
 from ryu.lib.packet.bgp import RF_IPv6_UC
 from ryu.lib.packet.bgp import RF_IPv4_VPN
 from ryu.lib.packet.bgp import RF_IPv6_VPN
+from ryu.lib.packet.bgp import RF_IPv4_FLOWSPEC
+from ryu.lib.packet.bgp import RF_VPNv4_FLOWSPEC
 from ryu.lib.packet.bgp import RF_RTC_UC
 from ryu.lib.packet.bgp import get_rf
 
@@ -75,6 +77,8 @@ from ryu.lib.packet.bgp import BGPPathAttributeAsPath
 from ryu.lib.packet.bgp import BGPPathAttributeAs4Path
 from ryu.lib.packet.bgp import BGPPathAttributeLocalPref
 from ryu.lib.packet.bgp import BGPPathAttributeExtendedCommunities
+from ryu.lib.packet.bgp import BGPPathAttributeOriginatorId
+from ryu.lib.packet.bgp import BGPPathAttributeClusterList
 from ryu.lib.packet.bgp import BGPPathAttributeMpReachNLRI
 from ryu.lib.packet.bgp import BGPPathAttributeMpUnreachNLRI
 from ryu.lib.packet.bgp import BGPPathAttributeCommunities
@@ -90,11 +94,16 @@ from ryu.lib.packet.bgp import BGP_ATTR_TYPE_MP_REACH_NLRI
 from ryu.lib.packet.bgp import BGP_ATTR_TYPE_MP_UNREACH_NLRI
 from ryu.lib.packet.bgp import BGP_ATTR_TYPE_MULTI_EXIT_DISC
 from ryu.lib.packet.bgp import BGP_ATTR_TYPE_COMMUNITIES
+from ryu.lib.packet.bgp import BGP_ATTR_TYPE_ORIGINATOR_ID
+from ryu.lib.packet.bgp import BGP_ATTR_TYPE_CLUSTER_LIST
 from ryu.lib.packet.bgp import BGP_ATTR_TYPE_EXTENDED_COMMUNITIES
+from ryu.lib.packet.bgp import BGP_ATTR_TYEP_PMSI_TUNNEL_ATTRIBUTE
 
 from ryu.lib.packet.bgp import BGPTwoOctetAsSpecificExtendedCommunity
 from ryu.lib.packet.bgp import BGPIPv4AddressSpecificExtendedCommunity
 
+from ryu.lib.packet import safi as subaddr_family
+
 LOG = logging.getLogger('bgpspeaker.peer')
 
 
@@ -249,7 +258,7 @@ class PeerState(object):
 
     @property
     def total_msg_recv(self):
-        """Returns total number of UPDATE, NOTIFCATION and ROUTE_REFRESH
+        """Returns total number of UPDATE, NOTIFICATION and ROUTE_REFRESH
         messages received from this peer.
         """
         return (self.get_count(PeerCounterNames.RECV_UPDATES) +
@@ -437,6 +446,10 @@ class Peer(Source, Sink, NeighborConfListener, Activity):
     def is_route_server_client(self):
         return self._neigh_conf.is_route_server_client
 
+    @property
+    def is_route_reflector_client(self):
+        return self._neigh_conf.is_route_reflector_client
+
     @property
     def check_first_as(self):
         return self._neigh_conf.check_first_as
@@ -975,9 +988,33 @@ class Peer(Source, Sink, NeighborConfListener, Activity):
                 new_pathattr.append(mpunreach_attr)
         elif self.is_route_server_client:
             nlri_list = [path.nlri]
-            for pathattr in path.pathattr_map.values():
-                new_pathattr.append(pathattr)
+            new_pathattr.extend(pathattr_map.values())
         else:
+            if self.is_route_reflector_client:
+                # Append ORIGINATOR_ID attribute if not already exist.
+                if BGP_ATTR_TYPE_ORIGINATOR_ID not in pathattr_map:
+                    originator_id = path.source
+                    if originator_id is None:
+                        originator_id = self._common_conf.router_id
+                    elif isinstance(path.source, Peer):
+                        originator_id = path.source.ip_address
+                    new_pathattr.append(
+                        BGPPathAttributeOriginatorId(value=originator_id))
+
+                # Preppend own CLUSTER_ID into CLUSTER_LIST attribute if exist.
+                # Otherwise append CLUSTER_LIST attribute.
+                cluster_lst_attr = pathattr_map.get(BGP_ATTR_TYPE_CLUSTER_LIST)
+                if cluster_lst_attr:
+                    cluster_list = list(cluster_lst_attr.value)
+                    if self._common_conf.cluster_id not in cluster_list:
+                        cluster_list.insert(0, self._common_conf.cluster_id)
+                    new_pathattr.append(
+                        BGPPathAttributeClusterList(cluster_list))
+                else:
+                    new_pathattr.append(
+                        BGPPathAttributeClusterList(
+                            [self._common_conf.cluster_id]))
+
             # Supported and un-supported/unknown attributes.
             origin_attr = None
             nexthop_attr = None
@@ -988,12 +1025,15 @@ class Peer(Source, Sink, NeighborConfListener, Activity):
             extcomm_attr = None
             community_attr = None
             localpref_attr = None
+            pmsi_tunnel_attr = None
             unknown_opttrans_attrs = None
             nlri_list = [path.nlri]
 
-            # By default, we use BGPSpeaker's interface IP with this peer
-            # as next_hop.
-            if self.is_ebgp_peer():
+            if path.route_family.safi in (subaddr_family.IP_FLOWSPEC,
+                                          subaddr_family.VPN_FLOWSPEC):
+                # Flow Specification does not have next_hop.
+                next_hop = []
+            elif self.is_ebgp_peer():
                 next_hop = self._session_next_hop(path)
                 if path.is_local() and path.has_nexthop():
                     next_hop = path.nexthop
@@ -1001,7 +1041,10 @@ class Peer(Source, Sink, NeighborConfListener, Activity):
                 next_hop = path.nexthop
                 # RFC 4271 allows us to change next_hop
                 # if configured to announce its own ip address.
-                if self._neigh_conf.is_next_hop_self:
+                # Also if the BGP route is configured without next_hop,
+                # we use path._session_next_hop() as next_hop.
+                if (self._neigh_conf.is_next_hop_self
+                        or (path.is_local() and not path.has_nexthop())):
                     next_hop = self._session_next_hop(path)
                     LOG.debug('using %s as a next_hop address instead'
                               ' of path.nexthop %s', next_hop, path.nexthop)
@@ -1114,8 +1157,10 @@ class Peer(Source, Sink, NeighborConfListener, Activity):
                 # For iBGP peers we are required to send local-pref attribute
                 # for connected or local prefixes. We check if the path matches
                 # attribute_maps and set local-pref value.
-                # If the path doesn't match, we set default local-pref 100.
-                localpref_attr = BGPPathAttributeLocalPref(100)
+                # If the path doesn't match, we set default local-pref given
+                # from the user. The default value is 100.
+                localpref_attr = BGPPathAttributeLocalPref(
+                    self._common_conf.local_pref)
                 key = const.ATTR_MAPS_LABEL_DEFAULT
 
                 if isinstance(path, (Vpnv4Path, Vpnv6Path)):
@@ -1164,7 +1209,11 @@ class Peer(Source, Sink, NeighborConfListener, Activity):
                     communities=communities
                 )
 
-            # UNKOWN Attributes.
+                pmsi_tunnel_attr = pathattr_map.get(
+                    BGP_ATTR_TYEP_PMSI_TUNNEL_ATTRIBUTE
+                )
+
+            # UNKNOWN Attributes.
             # Get optional transitive path attributes
             unknown_opttrans_attrs = bgp_utils.get_unknown_opttrans_attr(path)
 
@@ -1192,6 +1241,8 @@ class Peer(Source, Sink, NeighborConfListener, Activity):
                 new_pathattr.append(community_attr)
             if extcomm_attr:
                 new_pathattr.append(extcomm_attr)
+            if pmsi_tunnel_attr:
+                new_pathattr.append(pmsi_tunnel_attr)
             if unknown_opttrans_attrs:
                 new_pathattr.extend(unknown_opttrans_attrs.values())
 
@@ -1203,7 +1254,7 @@ class Peer(Source, Sink, NeighborConfListener, Activity):
         return update
 
     def _connect_loop(self, client_factory):
-        """In the current greeenlet we try to establish connection with peer.
+        """In the current greenlet we try to establish connection with peer.
 
         This greenlet will spin another greenlet to handle incoming data
         from the peer once connection is established.
@@ -1379,7 +1430,7 @@ class Peer(Source, Sink, NeighborConfListener, Activity):
             else:
                 yield L
         opts = list(flatten(
-            list(self._neigh_conf.get_configured_capabilites().values())))
+            list(self._neigh_conf.get_configured_capabilities().values())))
         open_msg = BGPOpen(
             my_as=asnum,
             bgp_identifier=bgpid,
@@ -1473,9 +1524,14 @@ class Peer(Source, Sink, NeighborConfListener, Activity):
                 raise bgp.MissingWellKnown(BGP_ATTR_TYPE_ORIGIN)
 
             # Validate Next hop.
-            # TODO(PH): Currently ignore other cases.
-            if (not mp_reach_attr.next_hop or
-                    (mp_reach_attr.next_hop == self.host_bind_ip)):
+            if mp_reach_attr.route_family.safi in (
+                    subaddr_family.IP_FLOWSPEC,
+                    subaddr_family.VPN_FLOWSPEC):
+                # Because the Flow Specification does not have nexthop,
+                # skips check.
+                pass
+            elif (not mp_reach_attr.next_hop or
+                  mp_reach_attr.next_hop == self.host_bind_ip):
                 LOG.error('Nexthop of received UPDATE msg. (%s) same as local'
                           ' interface address %s.',
                           mp_reach_attr.next_hop,
@@ -1495,36 +1551,42 @@ class Peer(Source, Sink, NeighborConfListener, Activity):
         Assumes Multiprotocol Extensions capability is supported and enabled.
         """
         assert self.state.bgp_state == const.BGP_FSM_ESTABLISHED
+
+        # Increment count of update received.
         self.state.incr(PeerCounterNames.RECV_UPDATES)
+
         if not self._validate_update_msg(update_msg):
             # If update message was not valid for some reason, we ignore its
             # routes.
             LOG.error('UPDATE message was invalid, hence ignoring its routes.')
             return
 
-        # Increment count of update received.
-        mp_reach_attr = update_msg.get_path_attr(BGP_ATTR_TYPE_MP_REACH_NLRI)
-        mp_unreach_attr = update_msg.get_path_attr(
-            BGP_ATTR_TYPE_MP_UNREACH_NLRI)
-
         # Extract advertised path attributes and reconstruct AS_PATH attribute
         self._extract_and_reconstruct_as_path(update_msg)
 
-        nlri_list = update_msg.nlri
-        withdraw_list = update_msg.withdrawn_routes
+        # Check if path attributes have loops.
+        if self._is_looped_path_attrs(update_msg):
+            return
 
+        umsg_pattrs = update_msg.pathattr_map
+        mp_reach_attr = umsg_pattrs.get(BGP_ATTR_TYPE_MP_REACH_NLRI, None)
         if mp_reach_attr:
-            # Extract advertised paths from given message.
+            # Extract advertised MP-BGP paths from given message.
             self._extract_and_handle_mpbgp_new_paths(update_msg)
 
+        mp_unreach_attr = umsg_pattrs.get(BGP_ATTR_TYPE_MP_UNREACH_NLRI, None)
         if mp_unreach_attr:
-            # Extract withdraws from given message.
+            # Extract MP-BGP withdraws from given message.
             self._extract_and_handle_mpbgp_withdraws(mp_unreach_attr)
 
+        nlri_list = update_msg.nlri
         if nlri_list:
+            # Extract advertised BGP paths from given message.
             self._extract_and_handle_bgp4_new_paths(update_msg)
 
+        withdraw_list = update_msg.withdrawn_routes
         if withdraw_list:
+            # Extract BGP withdraws from given message.
             self._extract_and_handle_bgp4_withdraws(withdraw_list)
 
     def _extract_and_reconstruct_as_path(self, update_msg):
@@ -1589,6 +1651,48 @@ class Peer(Source, Sink, NeighborConfListener, Activity):
                 as_path = self._construct_as_path_attr(as_path, as4_path)
                 update_msg.path_attributes.append(as_path)
 
+    def _is_looped_path_attrs(self, update_msg):
+        """
+        Extracts path attributes from the given UPDATE message and checks
+        if the given attributes have loops or not.
+
+        :param update_msg: UPDATE message instance.
+        :return: True if attributes have loops. Otherwise False.
+        """
+        umsg_pattrs = update_msg.pathattr_map
+        recv_open_msg = self.protocol.recv_open_msg
+
+        # Check if AS_PATH has loops.
+        aspath = umsg_pattrs.get(BGP_ATTR_TYPE_AS_PATH)
+        if (aspath is not None
+                and aspath.has_local_as(
+                    self.local_as,
+                    max_count=self._common_conf.allow_local_as_in_count)):
+            LOG.error(
+                'AS_PATH on UPDATE message has loops. '
+                'Ignoring this message: %s',
+                update_msg)
+            return
+
+        # Check if ORIGINATOR_ID has loops. [RFC4456]
+        originator_id = umsg_pattrs.get(BGP_ATTR_TYPE_ORIGINATOR_ID, None)
+        if (originator_id
+                and recv_open_msg.bgp_identifier == originator_id):
+            LOG.error(
+                'ORIGINATOR_ID on UPDATE message has loops. '
+                'Ignoring this message: %s',
+                update_msg)
+            return
+
+        # Check if CLUSTER_LIST has loops. [RFC4456]
+        cluster_list = umsg_pattrs.get(BGP_ATTR_TYPE_CLUSTER_LIST, None)
+        if (cluster_list
+                and self._common_conf.cluster_id in cluster_list.value):
+            LOG.error(
+                'CLUSTER_LIST on UPDATE message has loops. '
+                'Ignoring this message: %s', update_msg)
+            return
+
     def _extract_and_handle_bgp4_new_paths(self, update_msg):
         """Extracts new paths advertised in the given update message's
          *MpReachNlri* attribute.
@@ -1603,23 +1707,8 @@ class Peer(Source, Sink, NeighborConfListener, Activity):
         processing.
         """
         umsg_pattrs = update_msg.pathattr_map
-
-        msg_rf = RF_IPv4_UC
-        # Check if this route family is among supported route families.
-        if msg_rf not in SUPPORTED_GLOBAL_RF:
-            LOG.info(('Received route for route family %s which is'
-                      ' not supported. Ignoring paths from this UPDATE: %s') %
-                     (msg_rf, update_msg))
-            return
-
-        aspath = umsg_pattrs.get(BGP_ATTR_TYPE_AS_PATH)
-        # Check if AS_PATH has loops.
-        if aspath.has_local_as(self.local_as):
-            LOG.error('Update message AS_PATH has loops. Ignoring this'
-                      ' UPDATE. %s', update_msg)
-            return
-
         next_hop = update_msg.get_path_attr(BGP_ATTR_TYPE_NEXT_HOP).value
+
         # Nothing to do if we do not have any new NLRIs in this message.
         msg_nlri_list = update_msg.nlri
         if not msg_nlri_list:
@@ -1676,16 +1765,6 @@ class Peer(Source, Sink, NeighborConfListener, Activity):
         processing.
         """
         msg_rf = RF_IPv4_UC
-        # Check if this route family is among supported route families.
-        if msg_rf not in SUPPORTED_GLOBAL_RF:
-            LOG.info(
-                (
-                    'Received route for route family %s which is'
-                    ' not supported. Ignoring withdraws form this UPDATE.'
-                ) % msg_rf
-            )
-            return
-
         w_nlris = withdraw_list
         if not w_nlris:
             # If this is EOR of some kind, handle it
@@ -1740,13 +1819,6 @@ class Peer(Source, Sink, NeighborConfListener, Activity):
                      (msg_rf, update_msg))
             return
 
-        aspath = umsg_pattrs.get(BGP_ATTR_TYPE_AS_PATH)
-        # Check if AS_PATH has loops.
-        if aspath.has_local_as(self.local_as):
-            LOG.error('Update message AS_PATH has loops. Ignoring this'
-                      ' UPDATE. %s', update_msg)
-            return
-
         if msg_rf in (RF_IPv4_VPN, RF_IPv6_VPN):
             # Check if we have Extended Communities Attribute.
             # TODO(PH): Check if RT_NLRI afi/safi will ever have this attribute
@@ -1776,6 +1848,7 @@ class Peer(Source, Sink, NeighborConfListener, Activity):
                 return
 
         next_hop = mpreach_nlri_attr.next_hop
+
         # Nothing to do if we do not have any new NLRIs in this message.
         msg_nlri_list = mpreach_nlri_attr.nlri
         if not msg_nlri_list:
@@ -1838,11 +1911,9 @@ class Peer(Source, Sink, NeighborConfListener, Activity):
         # Check if this route family is among supported route families.
         if msg_rf not in SUPPORTED_GLOBAL_RF:
             LOG.info(
-                (
-                    'Received route for route family %s which is'
-                    ' not supported. Ignoring withdraws form this UPDATE.'
-                ) % msg_rf
-            )
+                'Received route family %s is not supported. '
+                'Ignoring withdraw routes on this UPDATE message.',
+                msg_rf)
             return
 
         w_nlris = mp_unreach_attr.withdrawn_routes
@@ -1929,8 +2000,8 @@ class Peer(Source, Sink, NeighborConfListener, Activity):
             # Open/Notification messages are currently handled by protocol and
             # nothing is done inside peer, so should not see them here.
             raise ValueError('Peer does not support handling of %s'
-                             ' message during % state' %
-                             (msg, self.state.bgp_state()))
+                             ' message during %s state' %
+                             (msg, self.state.bgp_state))
 
     def _handle_err_sor_msg(self, afi, safi):
         # Check if ERR capability is enabled for this peer.
@@ -2143,7 +2214,7 @@ class Peer(Source, Sink, NeighborConfListener, Activity):
             return
 
         # If this peer is a route server client, we forward the path
-        # regardless of AS PATH loop, whether the connction is iBGP or eBGP,
+        # regardless of AS PATH loop, whether the connection is iBGP or eBGP,
         # or path's communities.
         if self.is_route_server_client:
             outgoing_route = OutgoingRoute(path)
@@ -2166,7 +2237,7 @@ class Peer(Source, Sink, NeighborConfListener, Activity):
 
         # If path from a bgp-peer is new best path, we share it with
         # all bgp-peers except the source peer and other peers in his AS.
-        # This is default JNOS setting that in JNOS can be disabled with
+        # This is default Junos setting that in Junos can be disabled with
         # 'advertise-peer-as' setting.
         elif (self != path.source or
               self.remote_as != path.source.remote_as):
@@ -2175,8 +2246,14 @@ class Peer(Source, Sink, NeighborConfListener, Activity):
             # routing information contained in that UPDATE message to other
             # internal peers (unless the speaker acts as a BGP Route
             # Reflector) [RFC4271].
-            if (self.remote_as == self._core_service.asn and
-                    self.remote_as == path.source.remote_as):
+            if (self.remote_as == self._core_service.asn
+                    and self.remote_as == path.source.remote_as
+                    and isinstance(path.source, Peer)
+                    and not path.source.is_route_reflector_client
+                    and not self.is_route_reflector_client):
+                LOG.debug(
+                    'Skipping sending iBGP route to iBGP peer %s AS %s',
+                    self.ip_address, self.remote_as)
                 return
 
             # If new best path has community attribute, it should be taken into
