@@ -18,13 +18,22 @@
 """
 from abc import abstractmethod
 import logging
-import netaddr
 import numbers
 
+import netaddr
+
+from ryu.lib import ip
+
 from ryu.lib.packet.bgp import RF_IPv4_UC
 from ryu.lib.packet.bgp import RF_IPv6_UC
 from ryu.lib.packet.bgp import RF_IPv4_VPN
 from ryu.lib.packet.bgp import RF_IPv6_VPN
+from ryu.lib.packet.bgp import RF_L2_EVPN
+from ryu.lib.packet.bgp import RF_IPv4_FLOWSPEC
+from ryu.lib.packet.bgp import RF_IPv6_FLOWSPEC
+from ryu.lib.packet.bgp import RF_VPNv4_FLOWSPEC
+from ryu.lib.packet.bgp import RF_VPNv6_FLOWSPEC
+from ryu.lib.packet.bgp import RF_L2VPN_FLOWSPEC
 from ryu.lib.packet.bgp import RF_RTC_UC
 from ryu.lib.packet.bgp import BGPOptParamCapabilityFourOctetAsNumber
 from ryu.lib.packet.bgp import BGPOptParamCapabilityEnhancedRouteRefresh
@@ -45,6 +54,12 @@ from ryu.services.protocols.bgp.rtconf.base import CAP_MBGP_IPV4
 from ryu.services.protocols.bgp.rtconf.base import CAP_MBGP_IPV6
 from ryu.services.protocols.bgp.rtconf.base import CAP_MBGP_VPNV4
 from ryu.services.protocols.bgp.rtconf.base import CAP_MBGP_VPNV6
+from ryu.services.protocols.bgp.rtconf.base import CAP_MBGP_EVPN
+from ryu.services.protocols.bgp.rtconf.base import CAP_MBGP_IPV4FS
+from ryu.services.protocols.bgp.rtconf.base import CAP_MBGP_IPV6FS
+from ryu.services.protocols.bgp.rtconf.base import CAP_MBGP_VPNV4FS
+from ryu.services.protocols.bgp.rtconf.base import CAP_MBGP_VPNV6FS
+from ryu.services.protocols.bgp.rtconf.base import CAP_MBGP_L2VPNFS
 from ryu.services.protocols.bgp.rtconf.base import CAP_REFRESH
 from ryu.services.protocols.bgp.rtconf.base import CAP_RTC
 from ryu.services.protocols.bgp.rtconf.base import compute_optional_conf
@@ -83,6 +98,7 @@ PASSWORD = 'password'
 IN_FILTER = 'in_filter'
 OUT_FILTER = 'out_filter'
 IS_ROUTE_SERVER_CLIENT = 'is_route_server_client'
+IS_ROUTE_REFLECTOR_CLIENT = 'is_route_reflector_client'
 CHECK_FIRST_AS = 'check_first_as'
 ATTRIBUTE_MAP = 'attribute_map'
 IS_NEXT_HOP_SELF = 'is_next_hop_self'
@@ -100,12 +116,19 @@ DEFAULT_CAP_MBGP_IPV4 = True
 DEFAULT_CAP_MBGP_IPV6 = False
 DEFAULT_CAP_MBGP_VPNV4 = False
 DEFAULT_CAP_MBGP_VPNV6 = False
+DEFAULT_CAP_MBGP_EVPN = False
+DEFAULT_CAP_MBGP_IPV4FS = False
+DEFAULT_CAP_MBGP_IPV6FS = False
+DEFAULT_CAP_MBGP_VPNV4FS = False
+DEFAULT_CAP_MBGP_VPNV6FS = False
+DEFAULT_CAP_MBGP_L2VPNFS = False
 DEFAULT_HOLD_TIME = 40
 DEFAULT_ENABLED = True
 DEFAULT_CAP_RTC = False
 DEFAULT_IN_FILTER = []
 DEFAULT_OUT_FILTER = []
 DEFAULT_IS_ROUTE_SERVER_CLIENT = False
+DEFAULT_IS_ROUTE_REFLECTOR_CLIENT = False
 DEFAULT_CHECK_FIRST_AS = False
 DEFAULT_IS_NEXT_HOP_SELF = False
 DEFAULT_CONNECT_MODE = CONNECT_MODE_BOTH
@@ -139,7 +162,7 @@ def validate_changes(changes):
 
 
 def valid_ip_address(addr):
-    if not netaddr.valid_ipv4(addr) and not netaddr.valid_ipv6(addr):
+    if not ip.valid_ipv4(addr) and not ip.valid_ipv6(addr):
         return False
     return True
 
@@ -201,6 +224,7 @@ def valid_prefix_filter(filter_):
     le = filter_.get('le', None)
     return PrefixFilter(prefix, policy, ge=ge, le=le)
 
+
 PREFIX_FILTER = 'prefix_filter'
 
 SUPPORTED_FILTER_VALIDATORS = {
@@ -253,16 +277,25 @@ def validate_attribute_maps(attribute_maps):
 
 @validate(name=IS_ROUTE_SERVER_CLIENT)
 def validate_is_route_server_client(is_route_server_client):
-    if is_route_server_client not in (True, False):
+    if not isinstance(is_route_server_client, bool):
         raise ConfigValueError(desc='Invalid is_route_server_client(%s)' %
                                is_route_server_client)
 
     return is_route_server_client
 
 
+@validate(name=IS_ROUTE_REFLECTOR_CLIENT)
+def validate_is_route_reflector_client(is_route_reflector_client):
+    if not isinstance(is_route_reflector_client, bool):
+        raise ConfigValueError(desc='Invalid is_route_reflector_client(%s)' %
+                                    is_route_reflector_client)
+
+    return is_route_reflector_client
+
+
 @validate(name=CHECK_FIRST_AS)
 def validate_check_first_as(check_first_as):
-    if check_first_as not in (True, False):
+    if not isinstance(check_first_as, bool):
         raise ConfigValueError(desc='Invalid check_first_as(%s)' %
                                check_first_as)
 
@@ -271,7 +304,7 @@ def validate_check_first_as(check_first_as):
 
 @validate(name=IS_NEXT_HOP_SELF)
 def validate_is_next_hop_self(is_next_hop_self):
-    if is_next_hop_self not in (True, False):
+    if not isinstance(is_next_hop_self, bool):
         raise ConfigValueError(desc='Invalid is_next_hop_self(%s)' %
                                is_next_hop_self)
 
@@ -302,13 +335,19 @@ class NeighborConf(ConfWithId, ConfWithStats):
                                    CAP_FOUR_OCTET_AS_NUMBER,
                                    CAP_MBGP_IPV4, CAP_MBGP_IPV6,
                                    CAP_MBGP_VPNV4, CAP_MBGP_VPNV6,
-                                   CAP_RTC, RTC_AS, HOLD_TIME,
+                                   CAP_RTC, CAP_MBGP_EVPN,
+                                   CAP_MBGP_IPV4FS, CAP_MBGP_VPNV4FS,
+                                   CAP_MBGP_IPV6FS, CAP_MBGP_VPNV6FS,
+                                   CAP_MBGP_L2VPNFS,
+                                   RTC_AS, HOLD_TIME,
                                    ENABLED, MULTI_EXIT_DISC, MAX_PREFIXES,
                                    ADVERTISE_PEER_AS, SITE_OF_ORIGINS,
                                    LOCAL_ADDRESS, LOCAL_PORT, LOCAL_AS,
                                    PEER_NEXT_HOP, PASSWORD,
                                    IN_FILTER, OUT_FILTER,
-                                   IS_ROUTE_SERVER_CLIENT, CHECK_FIRST_AS,
+                                   IS_ROUTE_SERVER_CLIENT,
+                                   IS_ROUTE_REFLECTOR_CLIENT,
+                                   CHECK_FIRST_AS,
                                    IS_NEXT_HOP_SELF, CONNECT_MODE])
 
     def __init__(self, **kwargs):
@@ -328,8 +367,20 @@ class NeighborConf(ConfWithId, ConfWithStats):
             CAP_MBGP_IPV6, DEFAULT_CAP_MBGP_IPV6, **kwargs)
         self._settings[CAP_MBGP_VPNV4] = compute_optional_conf(
             CAP_MBGP_VPNV4, DEFAULT_CAP_MBGP_VPNV4, **kwargs)
+        self._settings[CAP_MBGP_EVPN] = compute_optional_conf(
+            CAP_MBGP_EVPN, DEFAULT_CAP_MBGP_EVPN, **kwargs)
         self._settings[CAP_MBGP_VPNV6] = compute_optional_conf(
             CAP_MBGP_VPNV6, DEFAULT_CAP_MBGP_VPNV6, **kwargs)
+        self._settings[CAP_MBGP_IPV4FS] = compute_optional_conf(
+            CAP_MBGP_IPV4FS, DEFAULT_CAP_MBGP_IPV4FS, **kwargs)
+        self._settings[CAP_MBGP_IPV6FS] = compute_optional_conf(
+            CAP_MBGP_IPV6FS, DEFAULT_CAP_MBGP_IPV6FS, **kwargs)
+        self._settings[CAP_MBGP_VPNV4FS] = compute_optional_conf(
+            CAP_MBGP_VPNV4FS, DEFAULT_CAP_MBGP_VPNV4FS, **kwargs)
+        self._settings[CAP_MBGP_VPNV6FS] = compute_optional_conf(
+            CAP_MBGP_VPNV6FS, DEFAULT_CAP_MBGP_VPNV6FS, **kwargs)
+        self._settings[CAP_MBGP_L2VPNFS] = compute_optional_conf(
+            CAP_MBGP_L2VPNFS, DEFAULT_CAP_MBGP_L2VPNFS, **kwargs)
         self._settings[HOLD_TIME] = compute_optional_conf(
             HOLD_TIME, DEFAULT_HOLD_TIME, **kwargs)
         self._settings[ENABLED] = compute_optional_conf(
@@ -345,6 +396,9 @@ class NeighborConf(ConfWithId, ConfWithStats):
         self._settings[IS_ROUTE_SERVER_CLIENT] = compute_optional_conf(
             IS_ROUTE_SERVER_CLIENT,
             DEFAULT_IS_ROUTE_SERVER_CLIENT, **kwargs)
+        self._settings[IS_ROUTE_REFLECTOR_CLIENT] = compute_optional_conf(
+            IS_ROUTE_REFLECTOR_CLIENT,
+            DEFAULT_IS_ROUTE_REFLECTOR_CLIENT, **kwargs)
         self._settings[CHECK_FIRST_AS] = compute_optional_conf(
             CHECK_FIRST_AS, DEFAULT_CHECK_FIRST_AS, **kwargs)
         self._settings[IS_NEXT_HOP_SELF] = compute_optional_conf(
@@ -492,6 +546,30 @@ class NeighborConf(ConfWithId, ConfWithStats):
     def cap_mbgp_vpnv6(self):
         return self._settings[CAP_MBGP_VPNV6]
 
+    @property
+    def cap_mbgp_evpn(self):
+        return self._settings[CAP_MBGP_EVPN]
+
+    @property
+    def cap_mbgp_ipv4fs(self):
+        return self._settings[CAP_MBGP_IPV4FS]
+
+    @property
+    def cap_mbgp_ipv6fs(self):
+        return self._settings[CAP_MBGP_IPV6FS]
+
+    @property
+    def cap_mbgp_vpnv4fs(self):
+        return self._settings[CAP_MBGP_VPNV4FS]
+
+    @property
+    def cap_mbgp_vpnv6fs(self):
+        return self._settings[CAP_MBGP_VPNV6FS]
+
+    @property
+    def cap_mbgp_l2vpnfs(self):
+        return self._settings[CAP_MBGP_L2VPNFS]
+
     @property
     def cap_rtc(self):
         return self._settings[CAP_RTC]
@@ -549,6 +627,10 @@ class NeighborConf(ConfWithId, ConfWithStats):
     def is_route_server_client(self):
         return self._settings[IS_ROUTE_SERVER_CLIENT]
 
+    @property
+    def is_route_reflector_client(self):
+        return self._settings[IS_ROUTE_REFLECTOR_CLIENT]
+
     @property
     def check_first_as(self):
         return self._settings[CHECK_FIRST_AS]
@@ -577,7 +659,7 @@ class NeighborConf(ConfWithId, ConfWithStats):
 
         return does_exceed
 
-    def get_configured_capabilites(self):
+    def get_configured_capabilities(self):
         """Returns configured capabilities."""
 
         capabilities = OrderedDict()
@@ -607,6 +689,36 @@ class NeighborConf(ConfWithId, ConfWithStats):
                 BGPOptParamCapabilityMultiprotocol(
                     RF_RTC_UC.afi, RF_RTC_UC.safi))
 
+        if self.cap_mbgp_evpn:
+            mbgp_caps.append(
+                BGPOptParamCapabilityMultiprotocol(
+                    RF_L2_EVPN.afi, RF_L2_EVPN.safi))
+
+        if self.cap_mbgp_ipv4fs:
+            mbgp_caps.append(
+                BGPOptParamCapabilityMultiprotocol(
+                    RF_IPv4_FLOWSPEC.afi, RF_IPv4_FLOWSPEC.safi))
+
+        if self.cap_mbgp_ipv6fs:
+            mbgp_caps.append(
+                BGPOptParamCapabilityMultiprotocol(
+                    RF_IPv6_FLOWSPEC.afi, RF_IPv6_FLOWSPEC.safi))
+
+        if self.cap_mbgp_vpnv4fs:
+            mbgp_caps.append(
+                BGPOptParamCapabilityMultiprotocol(
+                    RF_VPNv4_FLOWSPEC.afi, RF_VPNv4_FLOWSPEC.safi))
+
+        if self.cap_mbgp_vpnv6fs:
+            mbgp_caps.append(
+                BGPOptParamCapabilityMultiprotocol(
+                    RF_VPNv6_FLOWSPEC.afi, RF_VPNv6_FLOWSPEC.safi))
+
+        if self.cap_mbgp_l2vpnfs:
+            mbgp_caps.append(
+                BGPOptParamCapabilityMultiprotocol(
+                    RF_L2VPN_FLOWSPEC.afi, RF_L2VPN_FLOWSPEC.safi))
+
         if mbgp_caps:
             capabilities[BGP_CAP_MULTIPROTOCOL] = mbgp_caps
 
@@ -631,7 +743,7 @@ class NeighborConf(ConfWithId, ConfWithStats):
                                      self.enabled)
 
     def __str__(self):
-        return 'Neighbor: %s' % (self.ip_address)
+        return 'Neighbor: %s' % self.ip_address
 
 
 class NeighborsConf(BaseConf):
@@ -707,6 +819,7 @@ class NeighborsConf(BaseConf):
 class NeighborConfListener(ConfWithIdListener, ConfWithStatsListener):
     """Base listener for change events to a specific neighbors' configurations.
     """
+
     def __init__(self, neigh_conf):
         super(NeighborConfListener, self).__init__(neigh_conf)
         neigh_conf.add_listener(NeighborConf.UPDATE_ENABLED_EVT,
